// Bird.java

/**
 * An abstract class, that should be extended any kind of bird in the simulation.
 * Supplies many torus related methods.
 * By overwriting the method <code> getNewAcceleration() </code> you can change
 * the moving behavior of the child bird.
 */
import ch.aplu.jgamegrid.*;
import java.awt.Color;

abstract class Bird extends Actor
{
  protected GGVector startVelocity;
  protected GGVector velocity;
  protected GGVector position;
  protected boolean borderCrossed;
  protected double velocityMagnitude;

  public Bird(GGVector startVelocity, String sprite, double velocityMagnitude)
  {
    super(true, sprite);
    this.startVelocity = startVelocity;
    this.velocity = startVelocity;
    this.velocityMagnitude = velocityMagnitude;
  }

  protected abstract GGVector getNewAcceleration();

  protected abstract Color getColor();

  protected abstract void tryToEat();

  public void reset()
  {
    position = new GGVector(getLocationStart().x, getLocationStart().y);
    velocity = startVelocity;
    setDirection(Math.toDegrees(startVelocity.getDirection()));
  }

  public GGVector getPosition()
  {
    return position.clone();
  }

  public GGVector getVelocity()
  {
    return velocity.clone();
  }

  protected GGVector toTorusPosition(GGVector position)
  {
    double x = position.x;
    double y = position.y;
    if (x < 0)
      x = x + getNbHorzCells();
    else if (x > getNbHorzCells() - 1)
      x = x - getNbHorzCells();
    if (y < 0)
      y = y + getNbVertCells();
    else if (y > getNbVertCells() - 1)
      y = y - getNbVertCells();
    return new GGVector(x, y);
  }

  protected GGVector getPositionVecDiffOnTorus(Bird bird)
  {
    GGVector delta = new GGVector();
    double mag;
    double magmin = SimFlock.maxro;
    int imin = 0;
    int jmin = 0;

    for (int i = -1; i < 2; i++)
    {
      for (int j = -1; j < 2; j++)
      {
        delta.x = i * getNbHorzCells();
        delta.y = j * getNbVertCells();
        mag = position.sub(bird.getPosition().add(delta)).magnitude();
        if (mag < magmin)
        {
          magmin = mag;
          imin = i;
          jmin = j;
        }
      }
    }
    delta.x = imin * getNbHorzCells();
    delta.y = jmin * getNbVertCells();
    return position.sub(bird.getPosition().add(delta));
  }

  public void act()
  {
    GGVector acceleration = getNewAcceleration();
    velocity = velocity.add(acceleration.mult(SimFlock.timeFactor));
    velocity.normalize();
    velocity = velocity.mult(velocityMagnitude);
    position = position.add(velocity.mult(SimFlock.timeFactor));
    position = toTorusPosition(position);
    Location location = new Location((int) position.x, (int) position.y);
    setDirection(Math.toDegrees(velocity.getDirection()));
    setLocation(location);

    tryToEat();
  //  drawTrace();
  }

  private void drawTrace()
  {
    if (SimFlock.drawTrace)
    {
      getBackground().setPaintColor(getColor());
      getBackground().drawPoint(getPixelLocation());
    }
  }
}