//-------------- class FlockBird 

import ch.aplu.jgamegrid.*;
import java.awt.Color;
import java.util.*;

public class FlockBird extends Bird
{
  public FlockBird(GGVector startVelocity, double vbird)
  {
    super(startVelocity, "sprites/bird.gif", vbird);
  }

  protected GGVector getNewAcceleration()  //Behavior of the bird: acceleration
  {
    GGVector distVec = new GGVector();
    GGVector cohDistSumVec = new GGVector(0, 0);
    GGVector sepDistSumVec = new GGVector(0, 0);
    GGVector escDistSumVec = new GGVector(0, 0);
    GGVector aligVelSumVec = new GGVector(0, 0);
    GGVector acceleration = new GGVector(0, 0);
    GGVector accCohesion = new GGVector(0, 0);
    GGVector accSeparation = new GGVector(0, 0);
    GGVector accAlignment = new GGVector(0, 0);
    GGVector accEscape = new GGVector(0, 0);
    int cohCounter = 0;
    int sepCounter = 0;
    int aligCounter = 0;
    int raptCounter = 0;
    ArrayList<Actor> neighbourBirds = gameGrid.getActors(FlockBird.class);
    neighbourBirds.remove(this);  // Remove self from list
    ArrayList<Actor> neighbourRaptors = gameGrid.getActors(Raptor.class);

    for (Actor neighbour : neighbourBirds)
    {
      FlockBird bird = (FlockBird) neighbour;
      distVec = getPositionVecDiffOnTorus(bird);

      if ((distVec.magnitude() <= SimFlock.roFb) && (distVec.magnitude() > SimFlock.rcrit))
      {
        cohDistSumVec.x += distVec.x;
        cohDistSumVec.y += distVec.y;
        cohCounter++;
      }
      if (distVec.magnitude() <= SimFlock.rcrit)
      {
        sepDistSumVec.x += distVec.x;
        sepDistSumVec.y += distVec.y;
        sepCounter++;
      }
      if (distVec.magnitude() <= SimFlock.roFb)
      {
        aligVelSumVec.x += bird.velocity.x;
        aligVelSumVec.y += bird.velocity.y;
        aligCounter++;
      }
    }

    for (Actor neighbour : neighbourRaptors)
    {
      Raptor raptor = (Raptor) neighbour;
      distVec = getPositionVecDiffOnTorus(raptor);
      if (distVec.magnitude() <= SimFlock.roFb)
      {
        escDistSumVec.x += distVec.x;
        escDistSumVec.y += distVec.y;
        raptCounter++;
      }
    }

    if (cohCounter != 0)
    {
      cohDistSumVec.mult(1 / cohCounter);
    }
    if (sepCounter != 0)
    {
      sepDistSumVec.mult(1 / sepCounter);
    }
    if (raptCounter != 0)
    {
      escDistSumVec.mult(1 / raptCounter);
    }
    if (aligCounter != 0)
    {
      aligVelSumVec.mult(1 / aligCounter);
    }
    accCohesion = cohDistSumVec.mult(-SimFlock.cohesionFactor);
    accSeparation = sepDistSumVec.mult(SimFlock.separationFactor);
    accAlignment = velocity.sub(aligVelSumVec).mult(-SimFlock.alignmentFactor);
    accEscape = escDistSumVec.mult(SimFlock.escapeFactor);
    if (raptCounter > 0)
    { //fleeing is most important!
      acceleration.x = accEscape.x;
      acceleration.y = accEscape.y;
    }
    else
    {
      acceleration.x = accCohesion.x + accSeparation.x + accAlignment.x;
      acceleration.y = accCohesion.y + accSeparation.y + accAlignment.y;
    }
    return acceleration;
  }

  protected Color getColor()
  {
    return Color.BLUE;
  }

  protected void tryToEat()
  {
    //A Flockbird doesn't need to eat
  }
}