// SimFlock.java

import ch.aplu.jgamegrid.*;
import java.util.*;

public class SimFlock extends GameGrid
{
// Change game parameters 
  private static final int height = 600;             // Number of cells: Height of playground
  private static final int width = 1000;             // Number of cells: Width of playground;
  private static final int nbFlockBirds = 200;       // Number of FlockBirds
  private static final int nbRaptors = 3;            // Number of raptors
  protected static final double maxro = Math.sqrt(width * width + height * height);   // Maximum Radius of observation
  private static final int randomSeed = 123;               // For initializing birds locations
  protected static final boolean drawTrace = true;  // Draw traces of FlockBirds/raptors
  protected static final double timeFactor = 4;     // Descrete Timestep for calculation of acc/vel/pos
  // Change behavior of birds 
  protected static final double magnBird = 1;       // Magnitude of velocity: FlockBirds
  protected static final double roFb = 50;//50;     // Radius of observation: FlockBirds
  protected static final double rcrit = 20;//20;    // Critical Radius: FlockBirds
  protected static final double cohesionFactor = 0.001;//0.001; 
  protected static final double separationFactor = 0.1;//0.05;
  protected static final double alignmentFactor = 0.05;//0.05;
  protected static final double escapeFactor = 0.001; //0.001;
  // Change behavior of raptors 
  protected static final double magnRaptor = 1.1;     // Magnitude of velocity: raptors
  protected static final double roR = maxro;          // Radius of observation: raptors
  protected static final double aggressionFactor = 0.001;

  public SimFlock()
  {
    super(width, height, 1, null, true);
    setSimulationPeriod(50);
    Random rand = new Random(randomSeed);

    for (int i = 0; i < nbFlockBirds; i++)  // Generate FlockBirds
    {
      GGVector startVelocity = new GGVector(magnBird, 0);
      startVelocity.rotate(rand.nextInt(360));
      FlockBird fb = new FlockBird(startVelocity, magnBird);
      addActor(fb, new Location(rand.nextInt(width), rand.nextInt(height)));
    }
    for (int i = 0; i < nbRaptors; i++)  // Generate Raptors
    {
      GGVector startVelocity = new GGVector(magnRaptor, 0);
      startVelocity.rotate(rand.nextInt(360));
      Raptor raptor = new Raptor(startVelocity, magnRaptor);
      addActor(raptor, new Location(rand.nextInt(width), rand.nextInt(height)));
    }
    show();
  }

  public void reset()
  {
    getBg().clear();
  }

  public static void main(String[] args)
  {
    new SimFlock();
  }
}



